import {
  select,
  selection,
  min,
  max,
  // event,
  scaleTime,
  scaleLinear,
  timeFormat,
  axisBottom,
  timeMonth,
  brushX,
  transition,
} from 'd3';

import { data } from './getData';

const render = () => {
  const width = window.innerWidth;
  const height = window.innerHeight;

  const timelinechart = select('#timelinechart').node().dataset;
  // console.log(timelinechart);

  const config = timelinechart.config;
	// console.log(config);
  const { gutter } = JSON.parse(config);
	
  const now = Date.now();

  const lanes = [];
  const times = [];

  data.forEach((value, index) => {
    lanes.push(value.label);

    value.times.forEach((v, i) => {
      v.lane = index;
    });
    times.push(value.times);
  });

  const laneLength = lanes.length;
  const items = [].concat.apply([], times);
	
  items.forEach((v, i) => {
    v.id = i;
  });

  const timeBegin = min(items, (d) => {
    return d.starting_time;
  });
  const timeEnd = max(items, (d) => {
    return d.ending_time;
  });

  const m = [25, 150, 15, 105], //top right bottom left
    w = width - m[1] - m[3],
    h = height - m[0] - m[2],
    miniHeight = 50, //laneLength * 12 + 50,
    mainHeight = h - miniHeight - 50;

  //scales
  // for the mini axis
  const x = scaleTime()
    .range([0, w])
    .domain([timeBegin, timeEnd]);
  // for the top axis
  const xTop = scaleTime()
    .range([0, w])
    .domain([timeBegin, timeEnd]);
  // for the item squares
  // identical to xTop, except it is a scaleLinear, not a scaleTime.
  const x1 = scaleLinear().range([0, w]);
  const y1 = scaleLinear()
    .range([0, mainHeight])
    .domain([0, laneLength]);
  const y2 = scaleLinear()
    .range([0, miniHeight])
    .domain([0, laneLength]);

  const xAxis = axisBottom(x)
    .ticks(timeMonth)
    .tickFormat((d) => timeFormat('%Y-%m')(d));

  const xAxisTop = axisBottom(xTop)
    .ticks(timeMonth)
    .tickFormat((d) => timeFormat('%Y-%m-%d')(d));

  const scaleFactor = (1 / (timeEnd - timeBegin)) * w;

  const chartWidth = w + m[1] + m[3];
  const chartHeight = h + m[0] + m[2];

  const chart = select('#timelinechart')
    .append('svg')
    .attr('width', chartWidth)
    .attr('height', chartHeight)
    .attr(
      'viewBox',
      '0 0 ' + chartWidth + ' ' + chartHeight
    )
    .attr('preserveAspectRatio', 'xMidYMid meet')
    .append('g')
    .attr('class', 'timelinechartg');

  // console.log(mainHeight);
  chart
    .append('defs')
    .append('clipPath')
    .attr('id', 'clip')
    .append('rect')
    .attr('width', w)
    .attr('height', mainHeight);

  const main = chart
    .append('g')
    .attr(
      'transform',
      'translate(' + m[3] + ',' + m[0] + ')'
    )
    .attr('width', w)
    .attr('height', mainHeight)
    .attr('class', 'main');
	
  const mini = chart
    .append('g')
    .attr(
      'transform',
      'translate(' + m[3] + ',' + (mainHeight + m[0]) + ')'
    )
    .attr('width', w)
    .attr('height', miniHeight)
    .attr('class', 'mini');

  const gX = chart
    .append('g')
    .attr('class', 'base axis')
    .attr(
      'transform',
      'translate(' +
        m[3] +
        ',' +
        (mainHeight + miniHeight) +
        ')'
    )
    .call(xAxis);

  gX.selectAll('.tick text');

  const div = select('body')
    .append('div')
    .attr('class', 'tooltip')
    .style('opacity', 0);

  const gXTop = chart
    .append('g')
    .attr('class', 'axis')
    .attr('transform', 'translate(' + m[3] + ',' + 0 + ')')
    .attr('font-size', 20)
    .call(xAxisTop);

  const defs = main.append('svg:defs');

  //main lanes and texts
  
  
  main
    .append('g')
    .attr('class', 'core-chart')
    .selectAll('.laneLines')
    .data(items)
    .enter()
    .append('line')
    .attr('x1', 0)
    .attr('y1', (d) => {
      return y1(d.lane);
    })
    .attr('x2', w)
    .attr('y2', (d) => {
      return y1(d.lane);
    })
    .attr('stroke', 'lightgray')
  	.attr('fill','blue');

  main
    .append('g')
    .attr('class', 'core-labels')
    .selectAll('.laneText')
    .data(lanes)
    .enter()
    .append('text')
    .text(function (d) {
      return d;
    })
    .attr('x', -100)
    .attr('y', function (d, i) {
      return y1(i + 0.5);
    })
    .attr('dy', '.75ex')
    .attr('text-anchor', 'start')
    .attr('class', 'laneText');

  const itemRects = main
    .append('g')
    .attr('clip-path', 'url(#clip)');

  const currentLine = main
    .append('line')
    .attr('class', 'currentLine')
    .attr('clip-path', 'url(#clip)');
  
  const brushed = (event) => {
    const selection = event.selection;
    const timeSelection = selection.map(x.invert, x);
    const minExtent = timeSelection[0];
    const maxExtent = timeSelection[1];

    drawBrush(minExtent, maxExtent);
  }

  const brush = brushX()
    .extent([
      [0, 0],
      [w, miniHeight],
    ])
    .on('brush', brushed);

  const gBrush = mini
    .append('g')
    .attr('class', 'x brush')
    .call(brush)
    // .selectAll("rect")
    //.attr("y", 1)
    //.attr("height", miniHeight - 1)
    .call(brush.move, x.range());

  function toDays(d) {
    d = d || 0;
    return d / 24 / 60 / 60 / 1000;
  }

  function toUTC(d) {
    if (!d || !d.getFullYear) return 0;
    return Date.UTC(
      d.getFullYear(),
      d.getMonth(),
      d.getDate()
    );
  }

  function daysBetween(d1, d2) {
    return toDays(toUTC(d2) - toUTC(d1));
  }

  function drawBrush(minExtent, maxExtent) {
    const visItems = items.filter(function (d) {
      return (
        d.starting_time < maxExtent &&
        d.ending_time > minExtent
      );
    });

    const toolTipDateFormat = timeFormat('%Y-%m');

    const days = daysBetween(minExtent, maxExtent);

    let tFormat = '%Y-%m';
    let tTick = 'timeMonth';
    if (days < 40) {
      tFormat = '%Y-%m-%d';
      tTick = 'timeWeek';
    }

    xAxisTop
      .ticks(d3[tTick])
      .tickFormat((d) => timeFormat(tFormat)(d));

    x1.domain([minExtent, maxExtent]);
    xTop.domain([minExtent, maxExtent]);
    gXTop.call(xAxisTop);

    currentLine
      .attr('x1', x1(now))
      .attr('x2', x1(now))
      .attr('y1', 0)
      .attr('y2', mainHeight);

    //update main item rects
    const rects = itemRects
      .selectAll('rect')
      .data(visItems, function (d) {
        return d.id;
      })
      .attr('x',  (d) => {
        return x1(d.starting_time);
      })
      .attr('width',  (d) => {
        return x1(d.ending_time) - x1(d.starting_time);
      });

    rects
      .enter()
      .append('rect')
      .attr('class', d => {
      	return ('miniItem ' + d.text);
      })
      .attr('x', (d) => {
        return x1(d.starting_time)
      })
      .attr('y', (d) => {
      	// console.log(gutter)
        return y1(d.lane + gutter)
      })
      .attr('width', (d) => {
        return x1(d.ending_time) - x1(d.starting_time);
      })
      .attr('height', (d) => {
        return y1(1 - 2 * gutter);
      })
      .on('mouseover',  (event, d) => {
      // console.log(event.pageX);
      // console.log(d);
        div
          .transition()
          .duration(200)
          .style('opacity', 0.9);
        div
          .html(
                      'Country: ' +
              d.text +
              '<br>' +
            'Start Time: ' +
              toolTipDateFormat(d.starting_time) +
              '<br>' +
              'End Time: ' +
              toolTipDateFormat(d.ending_time)
          )
          .style('left', (event.pageX) + 'px')
          .style('top', (event.pageY) - 28 + 'px')
      		.style('height', '9%')
      		.style('width', '22%');

      })
      .on('mouseout', (d) => {
        div.transition().duration(500).style('opacity', 0);
      });

    rects.exit().remove();
  }



  gBrush
    .transition(transition().duration(750))
    .delay(500)
    .call(brush.move, [w * 0.2, w * 0.8]);

  //drawBrush(timeBegin, timeEnd);

  //console.log("mapppp", [0, 1].map(x1))

  //gBrush.call(brush.move, [0, 1].map(x1))

  //gBrush.call(brush.move, [0.3, 0.5].map(x1));
};
select(window).on('load', render);
// select('body').text(data[0]['label']);
